//
//  OrientationPlugin+Hook.m
//  Runner
//
//  Created by 蒋俊 on 2023/9/13.
//

#import "OrientationPlugin+Hook.h"
#import <objc/runtime.h>

@implementation OrientationPlugin (Hook)
+ (void)load
{
    Method originalMethod = class_getInstanceMethod(self, NSSelectorFromString(@"forceOrientation:"));
    Method swizzledMethod = class_getInstanceMethod(self, @selector(hook_forceOrientation:));
    method_exchangeImplementations(originalMethod, swizzledMethod);
}

- (void)hook_forceOrientation:(NSString *)orientation {
    // 若想在 iOS 16 上强制旋转生效
    // 1.请在 AppDelegate 中加入如下代码，允许所有旋转方向
    // 2.在Xcode中，选中General> Targets下的“Requires Full Screen”复选框，
    //    - (UIInterfaceOrientationMask)application:(UIApplication *)application supportedInterfaceOrientationsForWindow:(UIWindow *)window {
    //
    //        return UIInterfaceOrientationMaskAll;
    //    }
    if (@available(iOS 16.0, *)) {
        UIWindowSceneGeometryPreferencesIOS *perference = [[UIWindowSceneGeometryPreferencesIOS alloc] init];
        if ([orientation isEqualToString:@"DeviceOrientation.portraitUp"]) {
            perference.interfaceOrientations = UIInterfaceOrientationMaskPortrait;
            [self requestGeometryUpdateWithPreferences: perference];
        } else if ([orientation isEqualToString:@"DeviceOrientation.portraitDown"]) {
            perference.interfaceOrientations = UIInterfaceOrientationMaskPortraitUpsideDown;
            [self requestGeometryUpdateWithPreferences: perference];
        } else if ([orientation isEqualToString:@"DeviceOrientation.landscapeLeft"]) {
            perference.interfaceOrientations = UIInterfaceOrientationMaskLandscapeLeft;
            [self requestGeometryUpdateWithPreferences: perference];
        } else if ([orientation isEqualToString:@"DeviceOrientation.landscapeRight"]) {
            perference.interfaceOrientations = UIInterfaceOrientationMaskLandscapeRight;
            [self requestGeometryUpdateWithPreferences: perference];
        } else {
            perference.interfaceOrientations = UIInterfaceOrientationMaskPortrait;
            [self requestGeometryUpdateWithPreferences: perference];
        }
    } else {
        if ([orientation isEqualToString:@"DeviceOrientation.portraitUp"]) {
            [[UIDevice currentDevice] setValue:@(UIInterfaceOrientationPortrait) forKey:@"orientation"];
        } else if ([orientation isEqualToString:@"DeviceOrientation.portraitDown"]) {
            [[UIDevice currentDevice] setValue:@(UIInterfaceOrientationPortraitUpsideDown) forKey:@"orientation"];
        } else if ([orientation isEqualToString:@"DeviceOrientation.landscapeLeft"]) {
            [[UIDevice currentDevice] setValue:@(UIInterfaceOrientationLandscapeLeft) forKey:@"orientation"];
        } else if ([orientation isEqualToString:@"DeviceOrientation.landscapeRight"]) {
            [[UIDevice currentDevice] setValue:@(UIInterfaceOrientationLandscapeRight) forKey:@"orientation"];
        } else {
            [[UIDevice currentDevice] setValue:@(UIInterfaceOrientationUnknown) forKey:@"orientation"];
        }
    }
}

- (void)requestGeometryUpdateWithPreferences:(UIWindowSceneGeometryPreferences *)geometryPreferences  API_AVAILABLE(ios(16.0)){
    UIWindowScene *windowScene = (UIWindowScene *)[[[UIApplication sharedApplication] connectedScenes] allObjects].firstObject;
    [UIViewController attemptRotationToDeviceOrientation];
    [windowScene requestGeometryUpdateWithPreferences: geometryPreferences
                                         errorHandler: ^( NSError  * _Nonnull error) {
        NSLog(@"error--%@", error);
    }];
    UIWindow *keyWindow = [UIApplication sharedApplication].keyWindow;
    if (keyWindow != NULL) {
        [keyWindow.rootViewController setNeedsUpdateOfSupportedInterfaceOrientations];
    }
}

@end
